# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '25/Oct/2018'
__author__ = 'Erdinç Yılmaz'

import os
from PySide6.QtCore import QPoint, Qt
from PySide6.QtWidgets import QGraphicsScene
from . import shared
from .nesneler.kutuphane_nesnesi import KutuphaneNesnesi


#######################################################################
class Eleman:
    # ---------------------------------------------------------------------
    def __init__(self, tip, adres):
        self.tip = tip
        self.adres = adres

    # ---------------------------------------------------------------------
    def __hash__(self):
        return hash(self.adres)

    # ---------------------------------------------------------------------
    def __eq__(self, other):
        if not isinstance(other, type(self)):
            return NotImplemented
        # burda None==None mevzuu olabilir ama bizde none adres yok.
        return self.adres == other.adres and self.tip == other.tip


########################################################################
class SahneKutuphane(QGraphicsScene):

    # ---------------------------------------------------------------------
    def __init__(self, aktif_sahne, sayfalar, tempDirPath, parent=None):
        super(SahneKutuphane, self).__init__(parent)

        self.aktif_sahnedeki_dosya_isimleri = []

        self.sayfalar = sayfalar
        self.aktif_sahne = aktif_sahne
        self.tempDirPath = tempDirPath

        self.sonZDeger = 0

        self.belge_olceginde_islem_yap = False

        self.suruklenmekte_olan_nesne = None

        self.desteklenen_tipler = [shared.IMAGE_ITEM_TYPE, shared.VIDEO_ITEM_TYPE, shared.DOSYA_ITEM_TYPE]

    # ---------------------------------------------------------------------
    def zDeger_arttir(self, nesne):
        self.sonZDeger += 1
        nesne.setZValue(self.sonZDeger)

    # ---------------------------------------------------------------------
    def dragEnterEvent(self, e):
        # print(e)
        e.ignore()

    # ---------------------------------------------------------------------
    def dragMoveEvent(self, e):
        # print(e)
        e.acceptProposedAction()

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):
        mousePos = event.buttonDownScenePos(Qt.MouseButton.LeftButton)
        # self.movingItem = self.itemAt(mousePos)
        self.suruklenmekte_olan_nesne = self.itemAt(mousePos, self.views()[0].transform())
        # if self.suruklenmekte_olan_nesne:  # and event.button() == Qt.LeftButton
        super(SahneKutuphane, self).mousePressEvent(event)

    # ---------------------------------------------------------------------
    def aktif_sahnedeki_linkli_dosya_adresleri(self):
        kume = set()

        for item in self.aktif_sahne.items():
            if item.type() in self.desteklenen_tipler:
                if not item.isEmbeded:
                    kume.add(Eleman(tip=item.type(), adres=item.filePathForSave))
        return kume

    # ---------------------------------------------------------------------
    def tum_sahnelerdeki_linkli_dosya_adresleri(self):
        kume = set()

        for sayfa in self.sayfalar:
            for item in sayfa.scene.items():
                if item.type() in self.desteklenen_tipler:
                    if not item.isEmbeded:
                        kume.add(Eleman(tip=item.type(), adres=item.filePathForSave))
        return kume

    # ---------------------------------------------------------------------
    def aktif_sahnedeki_gomulu_dosya_adresleri(self):
        kume = set()

        for item in self.aktif_sahne.items():
            if item.type() in self.desteklenen_tipler:
                if item.isEmbeded:
                    kume.add(Eleman(tip=item.type(), adres=item.filePathForSave))
        return kume

    # ---------------------------------------------------------------------
    def tum_sahnelerdeki_gomulu_dosya_adresleri(self):
        """ bu belgedeki butun gomulu olanlardan farklı """
        # yukardakileri kullanmiyoruz cunku her bir sayfa icin yeni kume olusturma falan
        # gerkesiz belki ama boyle daha hizli
        kume = set()

        for sayfa in self.sayfalar:
            for item in sayfa.scene.items():
                if item.type() in self.desteklenen_tipler:
                    if item.isEmbeded:
                        kume.add(Eleman(tip=item.type(), adres=item.filePathForSave))
            if sayfa.view.backgroundImagePathIsEmbeded:
                kume.add(Eleman(tip=shared.IMAGE_ITEM_TYPE, adres=sayfa.view.backgroundImagePath))
        return kume

    # ---------------------------------------------------------------------
    def diskteki_tum_gomulu_dosya_adresleri(self):
        self.belge_olceginde_islem_yap = True
        # bunu inite tasima sayfa ve dokuman degistikce self.tempDirPath degisiyor.
        # self yapilcaksa, self.tempDirPath property yapilmali
        resimler_klasoru = os.path.join(self.tempDirPath, "images", "")
        videolar_klasoru = os.path.join(self.tempDirPath, "videos", "")
        dosyalar_klasoru = os.path.join(self.tempDirPath, "files", "")

        kume = set()
        try:
            for f in os.listdir(resimler_klasoru):
                kume.add((Eleman(tip=shared.IMAGE_ITEM_TYPE,
                                 adres=os.path.join(resimler_klasoru, f))))
        except FileNotFoundError:
            pass
            # print("FileNotFoundError: ", resimler_klasoru)

        try:
            for f in os.listdir(videolar_klasoru):
                kume.add((Eleman(tip=shared.VIDEO_ITEM_TYPE,
                                 adres=os.path.join(videolar_klasoru, f))))
        except FileNotFoundError:
            pass
            # print("FileNotFoundError: ", videolar_klasoru)

        try:
            for f in os.listdir(dosyalar_klasoru):
                kume.add((Eleman(tip=shared.DOSYA_ITEM_TYPE,
                                 adres=os.path.join(dosyalar_klasoru, f))))
        except FileNotFoundError:
            pass
            # print("FileNotFoundError: ", dosyalar_klasoru)

        return kume

    # ---------------------------------------------------------------------
    def diskteki_html_imaj_adresleri(self):
        self.belge_olceginde_islem_yap = True
        # bunu inite tasima sayfa ve dokuman degistikce self.tempDirPath degisiyor.
        # self yapilcaksa, self.tempDirPath property yapilmali
        images_html_klasoru = os.path.join(self.tempDirPath, "images-html", "")

        kume = set()
        try:
            for f in os.listdir(images_html_klasoru):
                kume.add(os.path.join(images_html_klasoru, f))
        except FileNotFoundError:
            pass
            # print("FileNotFoundError: ", images_html_klasoru)
        return kume

    # ---------------------------------------------------------------------
    def sahnedeki_linkli_dosyalari_goster(self):
        self.belge_olceginde_islem_yap = False
        self.clear()

        self._dosyalari_goster(self.aktif_sahnedeki_linkli_dosya_adresleri(),
                               isEmbeded=False)

    # ---------------------------------------------------------------------
    def sahnedeki_gomulu_dosyalari_goster(self):
        self.belge_olceginde_islem_yap = False
        self.clear()

        self._dosyalari_goster(self.aktif_sahnedeki_gomulu_dosya_adresleri(),
                               isEmbeded=True)

    # ---------------------------------------------------------------------
    def belgedeki_linkli_dosyalari_goster(self):
        self.belge_olceginde_islem_yap = True
        self.clear()

        self._dosyalari_goster(self.tum_sahnelerdeki_linkli_dosya_adresleri(),
                               isEmbeded=False)

    # ---------------------------------------------------------------------
    def belgedeki_gomulu_dosyalari_goster(self):
        self.belge_olceginde_islem_yap = True
        self.clear()
        pos = QPoint(5, 5)
        sayac = 1

        diskteki_tum_gomulu_dosya_adresleri_kumesi = self.diskteki_tum_gomulu_dosya_adresleri()
        toplam_kume_tum_sahneler = self.tum_sahnelerdeki_gomulu_dosya_adresleri()
        toplam_kume_aktif_sahne = self.aktif_sahnedeki_gomulu_dosya_adresleri()

        for eleman in diskteki_tum_gomulu_dosya_adresleri_kumesi:
            if eleman in toplam_kume_tum_sahneler:
                belgede_kullaniliyor_mu = True
            else:
                belgede_kullaniliyor_mu = False
            if eleman in toplam_kume_aktif_sahne:
                sahnede_kullaniliyor_mu = True
            else:
                sahnede_kullaniliyor_mu = False

            kutuphane_nesnesi = KutuphaneNesnesi(pos=pos, tip=eleman.tip,
                                                 dosya_adresi=eleman.adres,
                                                 sahnede_kullaniliyor_mu=sahnede_kullaniliyor_mu,
                                                 belgede_kullaniliyor_mu=belgede_kullaniliyor_mu,
                                                 isEmbeded=True)
            self.addItem(kutuphane_nesnesi)
            pos.setX(pos.x() + 50)
            if sayac % 5 == 0:
                pos.setX(5)
                pos.setY(pos.y() + 50)
            sayac += 1

        self.setSceneRect(0, 0, 250, pos.y())

    # ---------------------------------------------------------------------
    def _dosyalari_goster(self, kume, isEmbeded, pozisyonSifirla=True):

        self.sonZDeger = 0

        if pozisyonSifirla:
            pos = QPoint(5, 5)
            self.enSonElemanPos = pos
        else:
            pos = self.enSonElemanPos
        sayac = 1
        for eleman in kume:
            kutuphane_nesnesi = KutuphaneNesnesi(pos=pos, tip=eleman.tip,
                                                 dosya_adresi=eleman.adres,
                                                 isEmbeded=isEmbeded)
            self.zDeger_arttir(kutuphane_nesnesi)
            self.addItem(kutuphane_nesnesi)
            pos.setX(pos.x() + 50)
            if sayac % 5 == 0:
                pos.setX(5)
                pos.setY(pos.y() + 50)
            sayac += 1

        self.enSonElemanPos = pos

        # self.setSceneRect(self.itemsBoundingRect())
        # self.views()[0].zoomToFit()
        self.setSceneRect(0, 0, 250, pos.y())

    # ---------------------------------------------------------------------
    def belgedeki_linkli_ve_gomulu_dosyalari_goster(self):
        self.belge_olceginde_islem_yap = True
        self.clear()

        self._dosyalari_goster(self.tum_sahnelerdeki_linkli_dosya_adresleri(),
                               isEmbeded=False, pozisyonSifirla=True)

        self._dosyalari_goster(self.tum_sahnelerdeki_gomulu_dosya_adresleri(),
                               isEmbeded=True, pozisyonSifirla=False)

    # ---------------------------------------------------------------------
    def sahnedeki_linkli_ve_gomulu_dosyalari_goster(self):
        self.belge_olceginde_islem_yap = False
        self.clear()

        self._dosyalari_goster(self.aktif_sahnedeki_linkli_dosya_adresleri(),
                               isEmbeded=False, pozisyonSifirla=True)

        self._dosyalari_goster(self.aktif_sahnedeki_gomulu_dosya_adresleri(),
                               isEmbeded=True, pozisyonSifirla=False)

    # ---------------------------------------------------------------------
    def sahnede_kullanilmayan_gomulu_dosyalari_goster(self):
        self.belge_olceginde_islem_yap = False

        self.clear()
        pos = QPoint(5, 5)
        sayac = 1

        diskteki_tum_gomulu_dosya_adresleri_kumesi = self.diskteki_tum_gomulu_dosya_adresleri()
        toplam_kume = self.aktif_sahnedeki_gomulu_dosya_adresleri()

        fark_liste = list(diskteki_tum_gomulu_dosya_adresleri_kumesi - toplam_kume)

        for eleman in fark_liste:
            kutuphane_nesnesi = KutuphaneNesnesi(pos=pos, tip=eleman.tip,
                                                 dosya_adresi=eleman.adres,
                                                 sahnede_kullaniliyor_mu=False)
            self.addItem(kutuphane_nesnesi)
            pos.setX(pos.x() + 50)
            if sayac % 5 == 0:
                pos.setX(5)
                pos.setY(pos.y() + 50)
            sayac += 1

        self.setSceneRect(0, 0, 250, pos.y())

    # ---------------------------------------------------------------------
    def belgede_kullanilmayan_gomulu_dosyalari_goster(self):
        self.belge_olceginde_islem_yap = True

        self.clear()
        pos = QPoint(5, 5)
        sayac = 1

        diskteki_tum_gomulu_dosya_adresleri_kumesi = self.diskteki_tum_gomulu_dosya_adresleri()
        toplam_kume = self.tum_sahnelerdeki_gomulu_dosya_adresleri()

        fark_liste = list(diskteki_tum_gomulu_dosya_adresleri_kumesi - toplam_kume)
        for eleman in fark_liste:
            kutuphane_nesnesi = KutuphaneNesnesi(pos=pos, tip=eleman.tip,
                                                 dosya_adresi=eleman.adres,
                                                 belgede_kullaniliyor_mu=False)
            self.addItem(kutuphane_nesnesi)
            pos.setX(pos.x() + 50)
            if sayac % 5 == 0:
                pos.setX(5)
                pos.setY(pos.y() + 50)
            sayac += 1

        self.setSceneRect(0, 0, 250, pos.y())

    # ---------------------------------------------------------------------
    def kullanilmayanlari_isaretle(self):

        if self.belge_olceginde_islem_yap:

            diskteki_tum_gomulu_dosya_adresleri_kumesi = self.diskteki_tum_gomulu_dosya_adresleri()
            toplam_kume = self.tum_sahnelerdeki_gomulu_dosya_adresleri()
            fark_liste = list(diskteki_tum_gomulu_dosya_adresleri_kumesi - toplam_kume)
            fark_liste_adresler = [eleman.adres for eleman in fark_liste]

            for nesne in self.items():
                if nesne.dosya_adresi in fark_liste_adresler:
                    nesne.belgede_kullaniliyor_mu = False
                    nesne.update()
        else:
            toplam_kume = self.aktif_sahnedeki_gomulu_dosya_adresleri()
            toplam_kume_adresler = [eleman.adres for eleman in toplam_kume]
            for nesne in self.items():
                if nesne.dosya_adresi in toplam_kume_adresler:
                    continue
                nesne.sahnede_kullaniliyor_mu = False
                nesne.update()

    # ---------------------------------------------------------------------
    def secilen_nesneleri_sil(self):
        toplam_kume = self.aktif_sahnedeki_gomulu_dosya_adresleri()

        adresler = [eleman.adres for eleman in toplam_kume]

        for nesne in self.selectedItems():
            if nesne.dosya_adresi in adresler:
                continue
            os.unlink(nesne.dosya_adresi)
            self.removeItem(nesne)

    # ---------------------------------------------------------------------
    def belgede_kullanilmayan_gomulu_dosyalari_listele(self):
        diskteki_tum_gomulu_dosya_adresleri_kumesi = self.diskteki_tum_gomulu_dosya_adresleri()

        toplam_kume = self.tum_sahnelerdeki_gomulu_dosya_adresleri()

        fark_liste = list(diskteki_tum_gomulu_dosya_adresleri_kumesi - toplam_kume)

        return fark_liste

    # ---------------------------------------------------------------------
    def belgede_kullanilmayan_gomulu_dosyalari_sil(self, liste):
        if liste:
            for eleman in liste:
                try:
                    os.remove(eleman.adres)
                    self.parent().log(f"Dosya silindi: {eleman.adres}", level=1, toLogOnly=True)
                except Exception as e:
                    msj = f"HATA: Dosya silinemedi : {eleman.adres} {e}"
                    self.parent().log(msj, level=3, toLogOnly=True)
                    print(msj)

            self.parent().log(f"{len(liste)} adet dosya başarı ile silindi", level=1)

    # ---------------------------------------------------------------------
    def belgedeki_html_imajlari_goster(self):
        self.belge_olceginde_islem_yap = True
        self.clear()
        pos = QPoint(5, 5)
        sayac = 1

        diskteki_html_imaj_adresleri = self.diskteki_html_imaj_adresleri()

        for html_imaj_adresi in diskteki_html_imaj_adresleri:
            kutuphane_nesnesi = KutuphaneNesnesi(pos=pos, tip=shared.IMAGE_ITEM_TYPE,
                                                 dosya_adresi=html_imaj_adresi,
                                                 isHtmlImage=True,
                                                 isEmbeded=True)
            self.addItem(kutuphane_nesnesi)
            pos.setX(pos.x() + 50)
            if sayac % 5 == 0:
                pos.setX(5)
                pos.setY(pos.y() + 50)
            sayac += 1

        self.setSceneRect(0, 0, 250, pos.y())



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '01/Jun/2018'
__author__ = 'Erdinç Yılmaz'

from PySide6.QtCore import Qt
from PySide6.QtGui import QColor
from .shared import kim, slugify


########################################################################
class Sayfa(object):
    # class Sayfa(QObject):
    sayfa_no = 0  # 0 kok sayfaya gidiyor

    # ---------------------------------------------------------------------
    def __init__(self, adi=None, ikon=None, scene=None, view=None, parent=None):
        # super(Sayfa, self).__init__(parent)
        # self.setParent(parent)
        self._ic_sayfalar = []

        # print("sayfa parent", parent)

        # self.sayfaParent = parent

        self._parent = parent
        self._scene = scene
        self._view = view
        self._ikon = ikon

        self._kim = kim(kac_basamak=16)

        self._no = Sayfa.sayfa_no
        self._adi = "{} - {}".format(adi, self._no)
        # @adi.setter da temizleniyor, bu ilk olusturma
        self._kayit_adi = "{} - {}".format(slugify(self._adi, False), self._kim)

        # print(self.adi)

        Sayfa.sayfa_no += 1

        # #TODO: bu parent sayfa mı kontrol etmek lazım. bu herhangi bir nesne tip gelirse de calışır aşağıki kod yoksa
        # bu olmaz modelin kok sayfasına ekleyemeyiz burdan, parent none ise..
        # if parent is not None:
        #     parent.sayfa_ekle(self)

    # @pyqtProperty(str)
    @property
    def adi(self):
        return self._adi

    @adi.setter
    def adi(self, adi):
        self._adi = adi
        self._kayit_adi = "{} - {}".format(slugify(self._adi, False), self._kim)

    # @pyqtProperty(str)
    @property
    def ikon(self):
        return self._ikon

    @ikon.setter
    def ikon(self, ikon):
        self._ikon = ikon

    # @pyqtProperty(Scene)
    @property
    def scene(self):
        return self._scene

    @scene.setter
    def scene(self, scene):
        self._scene = scene

    # @pyqtProperty(View)
    @property
    def view(self):
        return self._view

    @view.setter
    def view(self, view):
        self._view = view

    # ---------------------------------------------------------------------
    def ustSayfa(self):
        return self._parent

    # ---------------------------------------------------------------------
    def parent(self):
        return self._parent

    # # ---------------------------------------------------------------------
    # def setUstSayfa(self, parent):
    #     self._parent = parent

    # ---------------------------------------------------------------------
    def ic_sayfa_var_mi(self):
        return bool(len(self._ic_sayfalar))

    # ---------------------------------------------------------------------
    def satirSayisi(self):
        return len(self._ic_sayfalar)

        # if self.parentItem:
        #     return self.parentItem.childItems.index(self)
        # return 0

    # ---------------------------------------------------------------------
    def sutunSayisi(self):
        # return len(self.itemData)
        return 1

    # ---------------------------------------------------------------------
    def ic_sayfalar(self):
        return self._ic_sayfalar

    # ---------------------------------------------------------------------
    def ic_sayfa(self, satir):
        # print(self.ic_sayfalar(), satir)
        # if self.ic_sayfalar():
        #     return self._ic_sayfalar[satir]
        return self._ic_sayfalar[satir]

    # ---------------------------------------------------------------------
    # def parent(self):
    #     return self.parent()

    # def __len__(self):
    #     return len(self._ic_sayfalar)

    # ---------------------------------------------------------------------
    def satir(self):  # row
        if self.parent():
            return self.parent().ic_sayfalar().index(self)  # bu index python liste indexi

        return 0  # kokSayfa

    # ---------------------------------------------------------------------
    def sutun(self):  # column
        return 1

    # # ---------------------------------------------------------------------
    # modelden silip eklemek , removeRow insertRow ile daha saglikli.
    # def sayfa_sira_degistir(self, sayfa, hedefSira):
    #     # sayfa = self._ic_sayfalar.pop(self._ic_sayfalar.index(sayfa))
    #     self._ic_sayfalar.remove(sayfa)
    #     self._ic_sayfalar.insert(hedefSira, sayfa)

    # ---------------------------------------------------------------------
    def sayfa_ekle(self, sayfa):
        # TODO: if not mantikli mi???
        if sayfa not in self._ic_sayfalar:
            self._ic_sayfalar.append(sayfa)
            # self._ic_sayfalar.append(sayfa)
            sayfa._parent = self
            # print(sayfa.ikon)
            return True

    #
    # def childAtRow(self, row):
    #     """The row-th child of this node."""
    #
    #     assert 0 <= row <= len(self.children)
    #     return self.children[row]
    #
    # def insertChild(self, child, position=0):
    #     """Insert a child in a group node."""
    #     self.children.insert(position, child)

    # ---------------------------------------------------------------------
    def sayfa_araya_ekle(self, satir, sayfa):
        # TODO: if not mantikli mi???
        if satir < 0 or satir > len(self._ic_sayfalar):
            return False

        self._ic_sayfalar.insert(satir, sayfa)
        sayfa._parent = self
        return True

    # ---------------------------------------------------------------------
    def sayfa_sil(self, sayfa):
        if sayfa in self._ic_sayfalar:
            self._ic_sayfalar.remove(sayfa)
            return True
        return False

    # ---------------------------------------------------------------------
    def sayfa_sil_sira_ile(self, sira):

        if sira < 0 or sira > len(self._ic_sayfalar):
            return False
        sayfa = self._ic_sayfalar.pop(sira)
        # sayfa.setParent(None)  # QObject oldugu icin
        # del sayfa
        return True

    # # ---------------------------------------------------------------------
    # def __eq__(self, other):
    #
    #     print(type(self), type(other))
    #     if other:
    #         return self.__dict__ == other.__dict__
    #     return False
    #
    # # ---------------------------------------------------------------------
    # def __ne__(self, other):
    #     if other is not self:
    #         return False
    #     return self.__dict__ != other.__dict__



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '8/1/15'

from PySide6.QtCore import (Qt, QRect, Signal, QSize)
from PySide6.QtGui import (QPainter, QPen)
from PySide6.QtWidgets import (QRubberBand, QWidget)


#######################################################################
class MyRubberBand(QRubberBand):

    # ---------------------------------------------------------------------
    def __init__(self, shape=QRubberBand.Rectangle, parent=None):
        super(MyRubberBand, self).__init__(shape, parent)
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.setWindowFlags(Qt.WindowType.FramelessWindowHint | Qt.WindowType.WindowStaysOnTopHint)
        # defaultFlags = self.windowFlags()
        # self.setWindowFlags(defaultFlags | Qt.ToolTip)
        # self.setWindowFlags(Qt.ToolTip)
        # self.setStyleSheet("selection-background-color: transparent")
        self._pen = QPen(Qt.GlobalColor.blue, 3, Qt.PenStyle.DotLine)

    # ---------------------------------------------------------------------
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setPen(self._pen)
        painter.setBrush(Qt.BrushStyle.NoBrush)
        painter.drawRect(event.rect().adjusted(0, 0, -1, -1))


#######################################################################
class TamEkranWidget_CM_On(QWidget):
    rubberBandReleased = Signal(QRect)
    esc_key_pressed = Signal()

    # ---------------------------------------------------------------------
    # def __init__(self, texts, parent=None):
    def __init__(self, is_compositing_manager_running=True, parent=None):

        super(TamEkranWidget_CM_On, self).__init__(parent)
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.setWindowFlags(Qt.WindowType.FramelessWindowHint | Qt.WindowType.WindowStaysOnTopHint)

        self.setAttribute(Qt.WidgetAttribute.WA_NoSystemBackground)
        self.setAttribute(Qt.WidgetAttribute.WA_TranslucentBackground, True)

        # self.setPalette(Qt.transparent)
        # self.setAttribute(Qt.WA_TransparentForMouseEvents)
        self.istek_mouse_releaseden_mi_geliyor = False
        self.rubberBand = None

        # self.setWindowOpacity(0.01)
        self.setWindowOpacity(0.7)
        # self.setStyleSheet("QWidget{background:transparent;}")
        # self.setStyleSheet("QWidget{background-color:none;}")
        self.showFullScreen()

    # ---------------------------------------------------------------------
    def closeEvent(self, event):
        if self.istek_mouse_releaseden_mi_geliyor:
            self.rubberBandReleased.emit(QRect(self.rubberBand.geometry()))
        event.accept()
        # super(TamEkranWidget_CM_Off, self).closeEvent(event)

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):
        self.origin = event.pos()

        if not self.rubberBand:
            # self.rubberBand = QRubberBand(QRubberBand.Rectangle)
            # self.rubberBand = MyRubberBand(QRubberBand.Rectangle)
            # TODO: bunu selfsiz de deneyelim compositor acık iken.
            self.rubberBand = MyRubberBand(QRubberBand.Shape.Rectangle, self)
            # self.rubberBand = MyRubberBand(QRubberBand.Rectangle)
        self.rubberBand.setGeometry(QRect(self.origin, QSize()))
        self.rubberBand.show()

        super(TamEkranWidget_CM_On, self).mousePressEvent(event)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):
        # self.rubberBand.setGeometry(QRect(self.origin, event.pos()).normalized())

        # self.rubberBand.setGeometry(QRect(self.mapToGlobal(self.origin),
        #                                   self.mapToGlobal(event.pos())).normalized())

        self.rubberBand.setGeometry(QRect(self.origin, event.pos()).normalized())

        # self.rubberBand.setGeometry(QRect(self.mapToGlobal(QCursor.pos()),
        #                                   self.mapToGlobal(event.pos())).normalized())

        super(TamEkranWidget_CM_On, self).mouseMoveEvent(event)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        self.rubberBand.hide()
        # self.rubberBand.close()
        # self.close()
        # self.setAttribute(Qt.WA_PaintOnScreen, False)
        # self.rubberBandReleased.emit(self.rubberBand.geometry())
        # super(TamEkranWidget_CM_On, self).mouseReleaseEvent(event)
        self.istek_mouse_releaseden_mi_geliyor = True
        self.close()
        self.deleteLater()

    # super(TamEkranWidget_CM_On, self).mouseReleaseEvent(event)
    # return QWidget.mouseReleaseEvent(event)

    # ---------------------------------------------------------------------
    def keyPressEvent(self, event):

        if event.key() == Qt.Key.Key_Escape:
            self.esc_key_pressed.emit()
        super(TamEkranWidget_CM_On, self).keyPressEvent(event)
        self.close()
        self.deleteLater()



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '5/4/16'

from PySide6.QtWidgets import QSlider, QProxyStyle
from PySide6.QtCore import Signal, Slot, Qt


#######################################################################
class ProxyStyle(QProxyStyle):
    """Gostergenin, fare sol tus ile tiklanan yere gitmesi icin. Yoksa adim adim(stepSize) gidiyor"""

    def styleHint(self, hint, opt=None, widget=None, returnData=None):
        res = super().styleHint(hint, opt, widget, returnData)
        if hint == QProxyStyle.StyleHint.SH_Slider_AbsoluteSetButtons:
            res |= Qt.MouseButton.LeftButton.value
        return res


#######################################################################
class Slider(QSlider):
    valueChangedFromSliderGuiNotBySetValue = Signal(int)

    # valueChangedWithSetValue = Signal(int)

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(Slider, self).__init__(parent)
        self.setStyle(ProxyStyle())
        self.kodIleSetEdiliyor = False
        self.valueChanged.connect(self.on_value_changed)

    # ---------------------------------------------------------------------
    def setValue(self, val):
        self.kodIleSetEdiliyor = True
        # self.setValue(val)
        QSlider.setValue(self, val)
        self.kodIleSetEdiliyor = False

    @Slot(int)
    def on_value_changed(self, val):
        if not self.kodIleSetEdiliyor:
            self.valueChangedFromSliderGuiNotBySetValue.emit(val)
        # else:
        #     self.valueChangedWithSetValue.emit(val)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/27/16'

import os
from PySide6.QtGui import QPainter, QPixmap, QPen
from PySide6.QtWidgets import QGraphicsView, QApplication
from PySide6.QtCore import Qt, QRectF, Slot


########################################################################
class View(QGraphicsView):
    def __init__(self, scene, parent=None):
        super(View, self).__init__(scene, parent)
        self.setDragMode(QGraphicsView.DragMode.RubberBandDrag)
        # self.setRenderHint(QPainter.Antialiasing)
        # self.setRenderHint(QPainter.TextAntialiasing)
        self.setRenderHints(QPainter.RenderHint.Antialiasing | QPainter.RenderHint.SmoothPixmapTransform)
        self.setRenderHint(QPainter.RenderHint.TextAntialiasing)
        # self.setDragMode(QGraphicsView.ScrollHandDrag)
        # self.setDragMode(QGraphicsView.RubberBandDrag)  # secmek icin meseala box selection
        # self.setRubberBandSelectionMode(Qt.ItemSelectionModelaraBak)

        # TODO !!
        self.setViewportUpdateMode(QGraphicsView.ViewportUpdateMode.FullViewportUpdate)
        # self.setViewportUpdateMode(QGraphicsView.ViewportUpdateMode.SmartViewportUpdate)
        # self.setViewportUpdateMode(self.MinimalViewportUpdate)  # default
        # self.setViewportUpdateMode(self.BoundingRectViewportUpdate)
        # self.setViewportUpdateMode(self.FullViewportUpdate)

        # self.setTransformationAnchor(QGraphicsView.AnchorUnderMouse)

        self.panStartPos = None

        # self.background = QPixmap('/pictures/rig.jpg')
        # self.backgroundImage = None

        self.setBackgroundBrush(Qt.GlobalColor.lightGray)
        self.backgroundImagePixmap = None
        self.backgroundImagePath = None
        self.backgroundImagePathIsEmbeded = False
        self.setCacheMode(QGraphicsView.CacheModeFlag.CacheBackground)

        self.setTransformationAnchor(QGraphicsView.ViewportAnchor.AnchorUnderMouse)
        self.baski_cerceve_kalemi = QPen(Qt.GlobalColor.black)
        self.baskiRectler = []

        # self.debug_rect=QRectF()

    # def scrollContentsBy(self, p_int, p_int_1):
    #     pass

    # def resizeEvent(self, QResizeEvent):
    #
    #     self.setSceneRect(self.get_visible_rect())
    #
    #     super(View, self).resizeEvent(QResizeEvent)

    # ---------------------------------------------------------------------
    def baski_cerceve_rengi_kur(self, renk):
        self.baski_cerceve_kalemi = QPen(renk,
                                         0,
                                         Qt.PenStyle.DashLine,
                                         Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)

    # ---------------------------------------------------------------------
    def setDragModeRubberBandDrag(self):
        self.setDragMode(QGraphicsView.DragMode.RubberBandDrag)

    # ---------------------------------------------------------------------
    def setDragModeNoDrag(self):
        self.setDragMode(QGraphicsView.DragMode.NoDrag)

    # ---------------------------------------------------------------------
    def contextMenuEvent(self, event):

        if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
            self.scene().parent().nesne_ozellikleriYW_goster_gizle()
            return

        if event.modifiers() == Qt.KeyboardModifier.AltModifier:
            self.scene().parent().stil_uygulaYW_goster_gizle()
            return

        QGraphicsView.contextMenuEvent(self, event)
        # mouse altinda bir nesne varsa ve sag click menusunu gosterdiyse event.accepted() doner.
        if not event.isAccepted():
            self.scene().parent().view_sag_menu_goster(event.globalPos())
            # TODO: gerek var mi?
            # event.accept()

    # ---------------------------------------------------------------------
    def get_visible_rect(self):
        """used for mirror line pos"""
        r = self.mapToScene(self.viewport().rect()).boundingRect()
        # r = self.mapToScene(self.rect()).boundingRect()
        r.adjust(1, 1, -1, -1)
        return r
        # return

    # # ---------------------------------------------------------------------
    # def get_scroll_bar_positions(self):
    #     return self.horizontalScrollBar().value(),self.verticalScrollBar().value()
    #
    # # ---------------------------------------------------------------------
    # def set_scroll_bar_positions(self, horizontal, vertical):
    #     self.horizontalScrollBar().setValue(horizontal)
    #     self.verticalScrollBar().setValue(vertical)

    # ---------------------------------------------------------------------
    def drawBackground(self, painter, rectf):
        painter.setBrushOrigin(0, 0)
        painter.fillRect(rectf, self.backgroundBrush())

        # ---  pos debug start   ---
        # painter.drawRect(self.sceneRect())
        # painter.drawRect(self.kopyaRekt)
        #
        # painter.drawEllipse(rectf.center().x(),rectf.center().y(),10,10)
        # #painter.setPen(QPen(Qt.blue))
        # #painter.drawRect(self.get_visible_rect())
        # painter.setPen(QPen(Qt.green))
        # painter.drawRect(self.scene().itemsBoundingRect())
        # painter.setPen(QPen(Qt.red))
        # painter.drawRect(QRectF(0,0,20,20))
        # painter.drawRect(self.sceneRect())
        # painter.setPen(QPen(Qt.yellow))
        # painter.drawRect(QRectF(0, 0, 20, 20))
        # painter.drawRect(QRectF(100, 0, 20, 20))
        # painter.drawRect(QRectF(200, 0, 20, 20))
        # painter.drawRect(QRectF(300, 0, 20, 20))
        # painter.drawRect(QRectF(400, 0, 20, 20))
        # painter.drawRect(QRectF(0, 100, 20, 20))
        # painter.drawRect(QRectF(0, 200, 20, 20))
        # painter.drawRect(QRectF(0, 300, 20, 20))
        # painter.drawRect(QRectF(0, 400, 20, 20))
        # painter.drawRect(self.kagitPozisyonBoyutRect)
        # painter.drawRect(self.debug_rect)
        # ---  pos debug end   ---

        if self.backgroundImagePixmap:
            # painter.drawPixmap(rectf.toRect(), self.backgroundImagePixmap)
            painter.drawPixmap(0, 0, self.backgroundImagePixmap)

        # painter.drawRect(QRectF(self.sceneRect().x(), self.sceneRect().y(), 10, 10))

        for rect in self.baskiRectler:
            painter.setPen(self.baski_cerceve_kalemi)
            painter.drawRect(rect)

    # ---------------------------------------------------------------------
    def set_background_image(self, imagePath):

        if not imagePath:
            self.backgroundImagePixmap = None
            self.backgroundImagePath = None
        else:
            # TODO: belki: ekelenen imajin ortalama rengine cevir scene bgcolor
            if not os.path.isfile(imagePath):
                imagePath = ':/icons/warning.png'
            self.backgroundImagePixmap = QPixmap(imagePath)
            # view's setSceneRect is not scene's setSceneRect, they are different.
            # self.setSceneRect(QRectF(self.backgroundImagePixmap.rect()))
            self.setSceneRect(QRectF(self.backgroundImagePixmap.rect()))
            self.fitInView(self.sceneRect(), Qt.AspectRatioMode.KeepAspectRatio)
            self.backgroundImagePath = imagePath
        self.resetCachedContent()

    # ---------------------------------------------------------------------
    def keyPressEvent(self, event):
        # self.scene().keyPressEvent(event)  # viewi bypass edip sahneye gonderiyoruz, ok tuslarini yiyor view

        key = event.key()
        # print(event.key())
        # print((Qt.Key_PageUp, Qt.Key_PageDown))

        # ---------------------------------------------------------------------
        # Bunu iptal ettik space basinca sahne hareket ettirmek icindi, keyreleasede de var devamı
        # ---------------------------------------------------------------------
        # if key == Qt.Key_Space:
        #     devam = False
        #     if self.scene().activeItem:
        #         if self.scene().activeItem.type() == shared.TEXT_ITEM_TYPE:
        #             if not self.scene().activeItem.hasFocus():
        #                 devam = True
        #         elif self.scene().activeItem.type() == shared.GROUP_ITEM_TYPE:
        #             # devam = False
        #             pass
        #         else:
        #             if not self.scene().activeItem.tempTextItem:
        #                 devam = True
        #     if devam:
        #         if not event.isAutoRepeat():
        #             if self.horizontalScrollBar().isVisible() or self.verticalScrollBar().isVisible():
        #                 self.setDragMode(QGraphicsView.ScrollHandDrag)
        #                 # bu interactive olmazsa mouse altında nesne olunca event nesneye geciyor.
        #                 self.setInteractive(False)
        #                 # self.bosluk_tusu_basildi_mi = True
        #                 # self.setCursor(Qt.ClosedHandCursor)
        #                 event.accept()
        # ---------------------------------------------------------------------

        # sahneye gitmeden once view' in abstractScrollAreasindaki
        # scrollbarlarin sliderSingleStepAdd aksiyonuna gidiyor. bu da nesne secili iken,
        #  ekranın scroll etmesine sebep oluyor
        if key in (Qt.Key.Key_Up, Qt.Key.Key_Down, Qt.Key.Key_Right, Qt.Key.Key_Left):
            if self.scene().activeItem:
                self.horizontalScrollBar().setSingleStep(0)
                self.verticalScrollBar().setSingleStep(0)
                # if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
                #     self.horizontalScrollBar().setPageStep(0)
                #     self.verticalScrollBar().setPageStep(0)
                # else:
                #     self.horizontalScrollBar().setSingleStep(0)
                #     self.verticalScrollBar().setSingleStep(0)

        # print(self.viewportEvent())/

        if key == Qt.Key.Key_Home:
            self.verticalScrollBar().setValue(self.verticalScrollBar().minimum())

        if key == Qt.Key.Key_End:
            self.verticalScrollBar().setValue(self.verticalScrollBar().maximum())

        if key == Qt.Key.Key_PageUp:
            self.verticalScrollBar().setValue(self.verticalScrollBar().value() - self.viewport().rect().height() + 10)

        if key == Qt.Key.Key_PageDown:
            self.verticalScrollBar().setValue(self.verticalScrollBar().value() + self.viewport().rect().height() - 10)

        super(View, self).keyPressEvent(event)

    # ---------------------------------------------------------------------
    def keyReleaseEvent(self, event):
        key = event.key()
        # ---------------------------------------------------------------------
        # Bunu iptal ettik space basinca sahne hareket ettirmek icindi, keypressde de var oncesi
        # ---------------------------------------------------------------------
        # if key == Qt.Key.Key_Space:
        #     if self.dragMode() == QGraphicsView.ScrollHandDrag:
        #         if not event.isAutoRepeat():
        #             self.setDragMode(QGraphicsView.RubberBandDrag)
        #             # bu interactive olmazsa mouse altında nesne olunca event nesneye geciyor.
        #             self.setInteractive(True)
        #             # self.bosluk_tusu_basildi_mi = False
        #             # self.setCursor(Qt.ArrowCursor)
        #             event.accept()
        # ---------------------------------------------------------------------

        super(View, self).keyPressEvent(event)

        # bunların superden sonra gelmesi lazim.
        if key in (Qt.Key.Key_Up, Qt.Key.Key_Down, Qt.Key.Key_Right, Qt.Key.Key_Left):
            # bir tusu birakmadan diger tusa basinca ve digerini birtakinca kaymalar oluyor o yuzden
            # secili nesne kontrolu
            if not self.scene().activeItem:
                self.horizontalScrollBar().setSingleStep(self.viewport().size().width() / 20)  # orjinal 1
                self.verticalScrollBar().setSingleStep(self.viewport().size().height() / 20)  # orjinal 1

            # self.horizontalScrollBar().setPageStep(self.viewport().size().width())
            # self.verticalScrollBar().setPageStep(self.viewport().size().height())

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):

        if event.button() == Qt.MouseButton.MiddleButton:
            # if not self.scene().selectedItems():
            # if not self.itemAt(event.pos()):
            self.panStartPos = event.pos()
            self.setCursor(Qt.CursorShape.ClosedHandCursor)
            event.accept()
            # nesne uzerine orta tıklayıp sahne hareket ettirisek,
            # orta tusu birakinca ic nesneleri seciyor ve secili birakiyor, o yuzden burda return kullandik
            # mouseReleaseEvent te de kullanabilirdik.
            return

        super(View, self).mousePressEvent(event)
        # return QGraphicsView.mousePressEvent(self, event)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):

        if event.button() == Qt.MouseButton.MiddleButton:
            self.setDragMode(QGraphicsView.DragMode.RubberBandDrag)
            self.panStartPos = None
            self.setCursor(Qt.CursorShape.ArrowCursor)
            self.setSceneRect(self.sceneRect().united(self.scene().itemsBoundingRect()))

            event.accept()
            # ! bunu mousePressEvent e tasidik.
            # nesne uzerine orta tıklayıp sahne hareket ettirisek,
            # orta tusu birakinca ic nesneleri seciyor ve secili birakiyor, o yuzden return
            # return

        super(View, self).mouseReleaseEvent(event)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):

        if self.panStartPos:
            # normal gezinme , sahne boyutlarini degistirmez. scroll bar sonlari sinirdir.
            # alt kisim eklenince, islevi biraz degisti, bu olmazsa sadece alt kisim varsa ve sahnede scrollbar varsa
            # sahnede gezinirken scrollbarlar bitene kadar sahne sabit kaliyor sonra gezinme basliyor.
            self.setDragMode(QGraphicsView.DragMode.ScrollHandDrag)
            self.horizontalScrollBar().setValue(
                self.horizontalScrollBar().value() - (event.x() - self.panStartPos.x()))
            self.verticalScrollBar().setValue(self.verticalScrollBar().value() - (event.y() - self.panStartPos.y()))

            # bu sonradan eklendi ve de bununla beraber,
            # mouse release eklendi- > self.setSceneRect(self.sceneRect().united(self.scene().itemsBoundingRect()))
            transform = self.transform()
            deltaX = (self.panStartPos.x() - event.x()) / transform.m11()
            deltaY = (self.panStartPos.y() - event.y()) / transform.m22()
            self.setSceneRect(self.sceneRect().translated(deltaX, deltaY))

            self.panStartPos = event.pos()

            event.accept()

        super(View, self).mouseMoveEvent(event)

    # ---------------------------------------------------------------------
    def wheelEvent(self, event):
        # factor = 1.41 ** (event.delta() / 240.0)
        # self.scale(factor, factor)

        if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
            # alt 2 satir olmazsa ctrl+shift nesnelere gecmiyor.
            if event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
                return QGraphicsView.wheelEvent(self, event)

            # alt 2 satir olmazsa ctrl+alt nesnelere gecmiyor.
            if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                return QGraphicsView.wheelEvent(self, event)

            if event.angleDelta().y() > 0:
                self.zoomIn()
            else:
                self.zoomOut()
            # return ediyoruz ki normal view ctrl hizli scroll iptal olsun
            return
        if event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
            #     # self.scene().tekerlek_ile_firca_boyu_degistir(event.angleDelta().y())
            #     # print(event.isAccepted())
            #     # if (event.isAccepted()):
            #     #     return
            #     # alt satir olmazsa shift nesnelere gecmiyor.

            # if not self.scene().activeItem:
            if self.itemAt(event.position().toPoint()):
                return QGraphicsView.wheelEvent(self, event)
        else:
            super(View, self).wheelEvent(event)
        # superi cagirmayalim cunku scrool bar move ediyor normalde wheel

    # # ---------------------------------------------------------------------
    # def wheelEvent_nesne_varsa_nesne_scale_eden(self, event):
    #     # factor = 1.41 ** (event.delta() / 240.0)
    #     # self.scale(factor, factor)
    #
    #     if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
    #         # alt satir olmazsa ctrl basili iken event nesnelere gecmiyor.
    #         if self.items(event.pos()):
    #             # for item in self.items(event.pos()):
    #             #     if item.isSelected
    #             # alt satir olmazsa ctrl basili iken event nesnelere gecmiyor.
    #             return QGraphicsView.wheelEvent(self, event)
    #         else:
    #
    #             # if event.delta() > 0:
    #             if event.angleDelta().y() > 0:
    #                 self.zoomIn()
    #             else:
    #                 self.zoomOut()
    #     else:
    #         super(View, self).wheelEvent(event)
    #     # superi cagirmayalim cunku scrool bar move ediyor normalde wheel

    # ---------------------------------------------------------------------
    @Slot(int)
    def set_scale(self, scale):
        """
            we set this from status bar spinbox
        """
        QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
        scale = scale / 100 * (1 / self.transform().m11())
        # self.scene().parent().zoomSBox.setValue(scale*100)
        self.scale(scale, scale)
        if self.scene().selectedItems():
            self.scene().activeItem.update_resize_handles()

        QApplication.restoreOverrideCursor()

    # ---------------------------------------------------------------------
    @Slot()
    def zoomIn(self):
        if self.scene().parent().zoomSBox.value() < 10000:
            QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
            self.scale(1.15, 1.15)
            # ir = self.scene().itemsBoundingRect()
            # vr = self.get_visible_rect()
            # if vr.width() * vr.height() > ir.width() * ir.height():
            #     self.setSceneRect(self.get_visible_rect())
            # else:
            #     self.setSceneRect(ir)
            if self.scene().selectedItems():
                self.scene().activeItem.update_resize_handles()

            self.scene().parent().zoomSBox.setValue(self.transform().m11() * 100)

            QApplication.restoreOverrideCursor()

    # ---------------------------------------------------------------------
    @Slot()
    def zoomOut(self):
        if self.scene().parent().zoomSBox.value() > 1:
            QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
            self.scale(1 / 1.15, 1 / 1.15)
            sr = self.scene().sceneRect()
            vr = self.get_visible_rect()
            if vr.width() * vr.height() > sr.width() * sr.height():
                self.setSceneRect(self.get_visible_rect())

            if self.scene().selectedItems():
                self.scene().activeItem.update_resize_handles()

            self.scene().parent().zoomSBox.setValue(self.transform().m11() * 100)
            QApplication.restoreOverrideCursor()

    # ---------------------------------------------------------------------
    @Slot()
    def zoomToFit(self):
        QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
        # make sure the sceneRect is only as large as it needs to be, since
        # it does not automatically shrink
        if self.scene().items():
            self.setSceneRect(self.scene().itemsBoundingRect())
            self.fitInView(self.scene().sceneRect(), Qt.AspectRatioMode.KeepAspectRatio)
            # self.fitInView(self.scene().itemsBoundingRect(), Qt.KeepAspectRatio)
        # self.setSceneRect(self.get_visible_rect())

        if self.scene().selectedItems():
            self.scene().activeItem.update_resize_handles()

        self.scene().parent().zoomSBox.setValue(self.transform().m11() * 100)
        QApplication.restoreOverrideCursor()

    # ---------------------------------------------------------------------
    @Slot()
    def zoomToSelection(self):
        QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
        if self.scene().selectionQueue:
            yeniRect = QRectF()
            for item in self.scene().selectionQueue:
                # if item.isVisible():
                if not item.parentItem():
                    yeniRect = yeniRect.united(item.boundingRect().translated(item.pos()))
                else:
                    # parent edilmis nesnelerin pos degerleri parenta gore
                    # bunlari sahneye goreye cevirmek lazim
                    itemPosMappedFromParent = item.mapFromParent(item.pos())
                    itemScenePos = item.mapToScene(itemPosMappedFromParent)
                    yeniRect = yeniRect.united(item.boundingRect().translated(itemScenePos))

            self.setSceneRect(yeniRect)
            self.debug_rect = yeniRect
            # self.fitInView(self.scene().sceneRect(), Qt.KeepAspectRatio)
            self.fitInView(yeniRect, Qt.AspectRatioMode.KeepAspectRatio)
            # self.setSceneRect(self.get_visible_rect())
            # self.centerOn(0, 0)

            # if self.scene().selectedItems():
            self.scene().activeItem.update_resize_handles()

            self.scene().parent().zoomSBox.setValue(self.transform().m11() * 100)
        QApplication.restoreOverrideCursor()

    # ---------------------------------------------------------------------
    @Slot()
    def zoomInitial(self):
        QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
        self.resetTransform()
        # sr = self.scene().sceneRect()
        ir = self.scene().itemsBoundingRect()
        vr = self.get_visible_rect()
        if vr.width() * vr.height() > ir.width() * ir.height():
            self.setSceneRect(self.get_visible_rect())
        else:
            self.setSceneRect(ir)

        if self.scene().selectedItems():
            self.scene().activeItem.update_resize_handles()

        self.scene().parent().zoomSBox.setValue(self.transform().m11() * 100)
        QApplication.restoreOverrideCursor()



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '04/Sep/2017'
__author__ = 'Erdinç Yılmaz'

from PySide6.QtCore import QSize, Qt, Signal
from PySide6.QtWidgets import QTreeWidget, QSizePolicy


#######################################################################
class TreeWidget(QTreeWidget):
    # tabWidgetFocusedIn = Signal()
    itemIsAboutToEdited = Signal(str)

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):

        super(TreeWidget, self).__init__(parent)

        self.setColumnCount(1)
        # self.setHeaderLabel("Sayfalar")
        # headerView = self.header()  # PyQt5.QtWidgets.QHeaderView
        self.header().hide()
        self.setSelectionBehavior(self.SelectionBehavior.SelectRows)
        self.setSelectionMode(self.SelectionMode.SingleSelection)
        self.setIconSize(QSize(128, 128))
        self.setUniformRowHeights(True)
        # self.setAnimated(True)
        self.setVerticalScrollMode(self.ScrollMode.ScrollPerPixel)
        self.verticalScrollBar().setSingleStep(15)

        # self.setStyleSheet("")

        # self.setStyleSheet(
        #     "QTreeView {show-decoration-selected: 1;}"
        #     "QTreeView::item {border: 1px solid #d9d9d9;"
        #     "    margin:0;padding:0;"
        #     "    border-top-color: transparent;"
        #     "    border-bottom-color: transparent;"
        #     "    border-top-color: black;border-bottom-color: black;"
        #     "}"
        #     "QTreeView::item:hover { background: qlineargradient(x1: 0, y1: 0," "x2: 0, y2: 1, stop: 0 #e7effd, stop: 1 #cbdaf1);"
        #     "    border: 1px solid #bfcde4;}"
        #     "QTreeView::item:selected {border: 1px solid #567dbc;}"
        #     "QTreeView::item:selected:active{background: qlineargradient(x1: 0," "y1: 0, x2: 0, y2: 1, stop: 0 #6ea1f1, stop: 1 #567dbc);}"
        #     "QTreeView::item:selected:!active {background: qlineargradient(x1:" "0, y1: 0, x2: 0, y2: 1, stop: 0 #6b9be8, stop: 1 #577fbf);}"
        #
        #     "QTreeView::branch:has-siblings:!adjoins-item {"
        #     "    border-image: url(canta/icons/stylesheet/vline.png) 0; }"
        #     "QTreeView::branch:has-siblings:adjoins-item {"
        #     "    border-image: url(canta/icons/stylesheet/branch-more.png) 0;}"
        #     "QTreeView::branch:!has-children:!has-siblings:adjoins-item {"
        #     "    border-image: url(canta/icons/stylesheet/branch-end.svg) 0;}"
        #     "QTreeView::branch:has-children:!has-siblings:closed,"
        #     "QTreeView::branch:closed:has-children:has-siblings {"
        #     "        border-image: none;"
        #     "        image: url(canta/icons/stylesheet/branch-closed.png);}"
        #     "QTreeView::branch:open:has-children:!has-siblings,"
        #     "QTreeView::branch:open:has-children:has-siblings  {"
        #     "        border-image: none;"
        #     "        image: url(canta/icons/stylesheet/branch-open.png);}"
        #                                         )

        self.setDragEnabled(True)
        self.setAcceptDrops(True)
        # asagidaki varken yukardakilere gerek yok, diger widgetler ile olan iliskiler ile ilgili yukardakiler.
        # TODO: liste elemani disari tasinabiliyor.
        self.setDragDropMode(self.DragDropMode.InternalMove)
        # pal = QPalette()

        # pal.setColor(QPalette.Background, Qt.white)
        # self.setPalette(pal)
        # self.setAutoFillBackground(True)
        self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        # self.sayfalarDW.setWidget(self.sayfalarDWBaseWidget)

    # ---------------------------------------------------------------------
    def dropEvent(self, e):
        # sayfadaki tek nesneyi drag drop edince secisisz kaliyor her sey.
        # ve secim degisince sayfa_degistirde current item none donuyor ve hata oluyor.
        # veya ctrl ile deselect edilebiliyor. selectionMode SingleItem olsa bile bu ctrl yi etkilemiyor.
        #
        # o yuzden burda agac da bir secisizlestirme olunca en bas nesne seciliyor. idi
        # onu yerine dragged itemi tekrar seciyoruz.
        # illa bir satir secili olmasi onemli,

        draggedItem = self.selectedItems()[0]

        sayfa = draggedItem.data(1, Qt.ItemDataRole.UserRole)[0]

        super(TreeWidget, self).dropEvent(e)

        if draggedItem.parent():
            yeniParent = draggedItem.parent().data(1, Qt.ItemDataRole.UserRole)[0]
        else:
            yeniParent = None
        # lol :)    [o.showMinimized() for o in QApplication.topLevelWidgets()]

        # print(QApplication.topLevelWidgets())
        self.parent().parent().parent().cDokuman.sayfa_tasi(sayfa,
                                                            yeniParent,
                                                            self.indexFromItem(draggedItem).row())

        if not self.selectedItems():
            draggedItem.setSelected(True)
            # self.setCurrentItem(self.topLevelItem(0), 0)

    # # ---------------------------------------------------------------------
    # def dragLeaveEvent(self, e):
    #     print("asd")
    #     e.ignore()
    #     # return
    #
    #     super(TreeWidget, self).dragLeaveEvent(e)

    # ---------------------------------------------------------------------
    def mousePressEvent(self, e):
        # to disable ctrl deselect
        super(TreeWidget, self).mousePressEvent(e)
        item = self.itemAt(e.pos())
        if item:  # item yazisina cift tiklayip edit edince hata oluyor. item yok diyor. o yuzden if kontrolu.
            item.setSelected(True)
            self.parent().parent().parent().tw_sayfa_degistir(item)

            # if item and item != not self.currentItem():
            #     self.clearSelection()
            #     item.setSelected(True)
            #     e.accept()

    # ---------------------------------------------------------------------
    def wheelEvent(self, event):
        # factor = 1.41 ** (event.delta() / 240.0)
        # self.scale(factor, factor)

        iconSize = self.iconSize()

        if event.modifiers() & Qt.KeyboardModifier.AltModifier:
            # if event.delta() > 0:
            if event.angleDelta().x() > 0:

                if not iconSize.width() >= 120:
                    # iconSize = 120
                    iconSize *= 1.1

                self.setIconSize(iconSize)
            else:

                if not iconSize.width() <= 16:
                    # iconSize = 16
                    iconSize *= .9
                self.setIconSize(iconSize)
        else:
            super(TreeWidget, self).wheelEvent(event)

    # ---------------------------------------------------------------------
    def mouseDoubleClickEvent(self, QMouseEvent):
        self.itemIsAboutToEdited.emit(self.currentItem().text(0))
        super(TreeWidget, self).mouseDoubleClickEvent(QMouseEvent)

    # ---------------------------------------------------------------------
    def keyPressEvent(self, QKeyEvent):
        if QKeyEvent.key() == Qt.Key.Key_F2:
            self.itemIsAboutToEdited.emit(self.currentItem().text(0))
        super(TreeWidget, self).keyPressEvent(QKeyEvent)

    # ---------------------------------------------------------------------
    def temizle(self):
        pass

    # ---------------------------------------------------------------------
    def sayfa_bul_ve_sec(self, sayfa):
        # https://doc.qt.io/qt-5/qt.html#MatchFlag-enum
        # self.findItems(textToMatch, Qt.MatchContains | Qt.MatchRecursive)
        self._sayfa_bul_ve_sec(self.invisibleRootItem(), sayfa)

    # ---------------------------------------------------------------------
    def _sayfa_bul_ve_sec(self, item, sayfa):

        for i in range(item.childCount()):
            child = item.child(i)
            if sayfa == child.data(1, Qt.ItemDataRole.UserRole)[0]:
                child.setSelected(True)
                self.setCurrentItem(child)
                break
            else:
                if child.childCount():
                    self._sayfa_bul_ve_sec(child, sayfa)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

import os
from unicodedata import normalize
from re import sub
from uuid import uuid4

from PySide6.QtCore import Qt, QRectF
from PySide6.QtGui import QColor, QPixmap, QPainter, QBrush, QTransform
from PySide6.QtWidgets import QGraphicsItem

DEFTER_KLASOR_ADRES = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
DEFTER_AYARLAR_DOSYA_ADRES = os.path.join(DEFTER_KLASOR_ADRES, "_ayarlar.ini")
DEFTER_AYARLAR_ARACLAR_DOSYA_ADRES = os.path.join(DEFTER_KLASOR_ADRES, "_ayarlar_araclar.ini")
DEFTER_ORG_NAME = "argekod"
DEFTER_ORG_DOMAIN = "argekod.com"
DEFTER_APP_NAME = "Defter"

renk_degismis_nesne_yazi_rengi = QColor(232, 50, 120)
# renk_degismis_nesne_yazi_rengi = QColor(0,0,255)
renk_kirmizi = QColor(Qt.GlobalColor.red)
renk_siyah = QColor(Qt.GlobalColor.black)
renk_mavi = QColor(Qt.GlobalColor.blue)
activeItemLineColor = QColor(Qt.GlobalColor.red)

# bunlarin degerleri degismesin, cunku nesneler kaydedilirken bu degerler de kaydediliyor.
# sonra eski dosyalar acilmaz, degisiklik yapilirsa
userType = QGraphicsItem.UserType
BASE_ITEM_TYPE = userType + 1
RECT_ITEM_TYPE = userType + 2
ELLIPSE_ITEM_TYPE = userType + 3
IMAGE_ITEM_TYPE = userType + 4
PATH_ITEM_TYPE = userType + 5
TEXT_ITEM_TYPE = userType + 6
WEB_ITEM_TYPE = userType + 7
VIDEO_ITEM_TYPE = userType + 8
GROUP_ITEM_TYPE = userType + 9
MIRRORLINE_TYPE = userType + 10
TEMP_TEXT_ITEM_TYPE = userType + 11
KUTUPHANE_NESNESI = userType + 12
LINE_ITEM_TYPE = userType + 13
DOSYA_ITEM_TYPE = userType + 14
YUVARLAK_FIRCA_BOYUTU_ITEM_TYPE = userType + 15


# ---------------------------------------------------------------------
def kim(kac_basamak):
    # veya str(uuid.uuid4())[:5] yerine uuid.uuid4().hex[:5]
    return uuid4().hex[:kac_basamak]


# ---------------------------------------------------------------------
def slugify(value, allow_unicode=False):
    # veya = "".join([x if x.isalnum() else "_" for x in value])
    # return veya
    """
    Taken from https://github.com/django/django/blob/master/django/utils/text.py
    Convert to ASCII if 'allow_unicode' is False. Convert spaces or repeated
    dashes to single dashes. Remove characters that aren't alphanumerics,
    underscores, or hyphens. Convert to lowercase. Also strip leading and
    trailing whitespace, dashes, and underscores.
    """
    value = str(value)
    if allow_unicode:
        value = normalize('NFKC', value)
    else:
        value = normalize('NFKD', value).encode('ascii', 'ignore').decode('ascii')
    value = sub(r'[^\w\s-]', '', value.lower())
    return sub(r'[-\s]+', '-', value).strip('-_')


# ---------------------------------------------------------------------
def unicode_to_bool(what):
    if what == "false" or what is False:
        return 0
    elif what == "true" or what is True:
        return 1


# ---------------------------------------------------------------------
def calculate_alpha(delta, col):
    alpha = col.alpha()

    if delta > 0:
        if alpha + 10 < 256:
            col.setAlpha(alpha + 10)
        else:
            col.setAlpha(255)
    else:
        if alpha - 10 > 0:
            col.setAlpha(alpha - 10)
        else:
            col.setAlpha(0)

    return col

    # return QColor().fromRgb(col.red(), col.green(),col.blue(),col.alpha())


# ---------------------------------------------------------------------
def kutulu_arkaplan_olustur(widget, kareBoyutu=10):
    kb = kareBoyutu

    pMap = QPixmap(2 * kb, 2 * kb)
    p = QPainter(pMap)
    p.fillRect(0, 0, kb, kb, QColor(200, 200, 200))
    p.fillRect(kb, kb, kb, kb, QColor(210, 210, 210))
    p.fillRect(0, kb, kb, kb, QColor(190, 190, 190))
    p.fillRect(kb, 0, kb, kb, QColor(200, 200, 200))
    p.end()
    palette = widget.palette()
    palette.setBrush(widget.backgroundRole(), QBrush(pMap))
    widget.setAutoFillBackground(True)
    widget.setPalette(palette)


# ---------------------------------------------------------------------
def _scaleChildItemsByResizing(nesne, scaleFactor):
    for c in nesne.childItems():
        if c.childItems():
            _scaleChildItemsByResizing(c, scaleFactor)

        if c.type() == TEXT_ITEM_TYPE:

            # c.setTransformOriginPoint(c.boundingRect().center())
            fontPointSizeF = c.fontPointSizeF() * scaleFactor
            c.setFontPointSizeF(fontPointSizeF)
            c.yazi_kutusunu_daralt()

        elif c.type() == GROUP_ITEM_TYPE:
            rect = QRectF(c._itemsBoundingRect.topLeft(), c._itemsBoundingRect.size() * scaleFactor)
            rect.moveTo(0, 0)
            c._itemsBoundingRect = rect

        elif c.type() == PATH_ITEM_TYPE:
            scaleMatrix = QTransform()
            scaleMatrix.scale(scaleFactor, scaleFactor)
            scaledPath = c.path() * scaleMatrix
            c.setPath(scaledPath)

            c.update_resize_handles()
            c.update_painter_text_rect()

        elif c.type() == LINE_ITEM_TYPE:
            scaleMatrix = QTransform()
            scaleMatrix.scale(scaleFactor, scaleFactor)
            scaledLine = c.line() * scaleMatrix
            c.setLine(scaledLine)

            c.update_resize_handles()
            c.update_painter_text_rect()

        else:
            rect = QRectF(c.rect().topLeft(), c.rect().size() * scaleFactor)
            rect.moveTo(0, 0)
            c.setRect(rect)

            c.update_resize_handles()
            c.update_painter_text_rect()

        c.setX(c.x() * scaleFactor)
        c.setY(c.y() * scaleFactor)


# ---------------------------------------------------------------------
def _update_scene_rect_recursively(items, rect):
    for c in items:
        rect = rect.united(c.sceneBoundingRect())
        if c.type() == GROUP_ITEM_TYPE:
            if c.parentedWithParentOperation:
                rect = _update_scene_rect_recursively(c.parentedWithParentOperation, rect)
        else:
            if c.childItems():
                rect = _update_scene_rect_recursively(c.childItems(), rect)

    return rect


# ---------------------------------------------------------------------
def sceneBoundingRectWithChildren(nesne):
    # rect = QRectF(self.sceneBoundingRect())
    rect = nesne.sceneBoundingRect()
    if nesne.type() == GROUP_ITEM_TYPE:
        ic_nesneler = nesne.parentedWithParentOperation
    else:
        ic_nesneler = nesne.childItems()
    for c in ic_nesneler:
        rect = rect.united(c.sceneBoundingRect())
        if c.type() == GROUP_ITEM_TYPE:
            if c.parentedWithParentOperation:
                rect = _update_scene_rect_recursively(c.parentedWithParentOperation, rect)
        else:
            if c.childItems():
                rect = _update_scene_rect_recursively(c.childItems(), rect)
    return rect



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '23/06/22'

from PySide6.QtCore import Qt, Signal
from PySide6.QtGui import QLinearGradient, QPainter
from .slider import Slider


#######################################################################
class SliderRenk(Slider):
    """  """
    degerDegisti = Signal(int)
    sliderClosed = Signal()

    # ----------------------------------------------------------------------
    def __init__(self, parent=None):
        """ Constructor """
        super(SliderRenk, self).__init__(parent)
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)

        self.valueChanged.connect(lambda x: self.degerDegisti.emit(x))

        # self.setFocusPolicy(Qt.NoFocus)
        # self.setFocusPolicy(Qt.WheelFocus)
        # self.setFocusPolicy(Qt.ClickFocus)

        self.setRange(0, 255)
        self.setOrientation(Qt.Orientation.Horizontal)

        self.setGeometry(0, 0, 150, 50)

        self.setStyleSheet(" QSlider::groove:horizontal "
                           "{border: 0px solid #999999;"
                           "height: 20px;} "
                           # " /* the groove expands to the size of the slider by default."
                           # " by giving it a height, it has a fixed size */ "
                           # "background: qlineargradient(x1:0, y1:0, x2:0, y2:1, } "
                           # "stop:0 #B1B1B1,}  "
                           # "stop:1 #c4c4c4);} "
                           # "margin: 2px 0;}  "

                           "QSlider::handle:horizontal "
                           "{ "
                           "background: qlineargradient(x1:0, y1:0, x2:1, y2:1, "
                           "stop:0 #fff, "
                           "stop:1 #00000000);  "
                           # "background:#fff;"
                           "border: 3px solid #fff;  "
                           "width: 7px;  "
                           "margin: -1px -5px; }"

                           )
        # "/* handle is placed by default on the contents rect of the groove."
        # " Expand outside the groove */ "
        # " border-radius: 0px; }")
        self.bgGradient = QLinearGradient(0, 0, 1, 0)
        self.bgGradient.setCoordinateMode(QLinearGradient.CoordinateMode.ObjectMode)

    # ---------------------------------------------------------------------
    def setBgGradient(self, bgGradient):
        self.bgGradient = bgGradient
        self.update()

    # ----------------------------------------------------------------------
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.fillRect(self.rect(), self.bgGradient)
        # painter.setBrush(self.bgGradient)
        # painter.drawRect(self.rect())

        super(SliderRenk, self).paintEvent(event)


#######################################################################
class SliderRenkYalniz(SliderRenk):
    """  """

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        """ Constructor """
        super(SliderRenkYalniz, self).__init__(parent)
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        # self.setFocusPolicy(Qt.NoFocus)
        self.setFocusPolicy(Qt.FocusPolicy.WheelFocus)
        # self.setFocusPolicy(Qt.ClickFocus)

    # ---------------------------------------------------------------------
    def focusOutEvent(self, event):
        # self.close()
        self.hide()
        # TODO bu signal kullanmamisiz. comment edelim..
        self.sliderClosed.emit()
        # print("focus out")
        super(SliderRenkYalniz, self).focusOutEvent(event)

    # ---------------------------------------------------------------------
    def keyPressEvent(self, event):
        if event.key() == Qt.Key.Key_Escape:
            # self.close()
            self.hide()
        super(SliderRenkYalniz, self).keyPressEvent(event)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '26/1/22'
__author__ = 'Erdinç Yılmaz'

from PySide6.QtCore import Signal
from PySide6.QtGui import QColor, Qt
from PySide6.QtWidgets import QWidget, QHBoxLayout, QVBoxLayout, QButtonGroup, QRadioButton, QLabel
from ..renkSecici import RenkSeciciWidget
from ..renkSecmeDugmesi import RenkSecmeDugmesi
from ..sliderDoubleWithDoubleSpinBox import SliderDoubleWithDoubleSpinBox
from ..yw.yuzenWidget import YuzenWidget


#######################################################################
class NesneOzellikleriYW(YuzenWidget):
    arkaPlanRengiDegisti = Signal(QColor)
    yaziRengiDegisti = Signal(QColor)
    cizgiRengiDegisti = Signal(QColor)
    ekranRenkSeciciDugmesindenRenkDegisti = Signal(QColor, str)
    cizgiKalinligiDegisti = Signal(float)

    # ---------------------------------------------------------------------
    def __init__(self, arkaPlanRengi, yaziRengi, cizgiRengi, cizgiKalinligi, parent=None):
        super(NesneOzellikleriYW, self).__init__(kapatilabilir_mi=True, parent=parent)

        self.arkaPlanRengi = arkaPlanRengi.toHsv()
        self.yaziRengi = yaziRengi.toHsv()
        self.cizgiRengi = cizgiRengi.toHsv()
        self.cizgiKalinligi = cizgiKalinligi
        # self.nesneBoyutu = nesneBoyutu

        # ilk acilista arkaplan rengi geliyor, sonra da kapatilinca gizlendigi icin devamlilik var.
        self.renk = self.arkaPlanRengi

        self.aracIkonuEtiketi = QLabel(self)

        self.ekrandaRenkSecmeDugmesi = RenkSecmeDugmesi(self)
        self.ekrandaRenkSecmeDugmesi.renkDegisti.connect(self.ekranRenkSeciciDugmesindenRenkDegisti.emit)

        self.radioBtnGroup = QButtonGroup(self)

        self.radioArkaplan = QRadioButton(self.tr("Background"), self)
        self.radioArkaplan.setFocusPolicy(Qt.FocusPolicy.TabFocus)
        self.radioYazi = QRadioButton(self.tr("Text"), self)
        self.radioYazi.setFocusPolicy(Qt.FocusPolicy.TabFocus)
        self.radioCizgi = QRadioButton(self.tr("Line && Pen"), self)
        self.radioCizgi.setFocusPolicy(Qt.FocusPolicy.TabFocus)

        self.radioBtnGroup.addButton(self.radioArkaplan)
        self.radioBtnGroup.addButton(self.radioYazi)
        self.radioBtnGroup.addButton(self.radioCizgi)

        radioLay = QHBoxLayout()

        radioLay.addSpacing(5)
        radioLay.addWidget(self.aracIkonuEtiketi)
        radioLay.addSpacing(3)
        radioLay.addWidget(self.ekrandaRenkSecmeDugmesi, 1)
        radioLay.addSpacing(5)
        radioLay.addWidget(self.radioArkaplan, 2)
        radioLay.addSpacing(5)
        radioLay.addWidget(self.radioYazi, 2)
        radioLay.addSpacing(5)
        radioLay.addWidget(self.radioCizgi, 2)

        self.radioArkaplan.setChecked(True)
        self.renk_tipi = "a"
        self.radioBtnGroup.buttonToggled.connect(self.act_radio_secim_degisti)

        self.cizgiKalinligiDSlider = SliderDoubleWithDoubleSpinBox(parent=self)
        # self.cizgiKalinligiDSlider.setMaximumWidth(150)
        self.cizgiKalinligiDSlider.setMinimum(0)
        self.cizgiKalinligiDSlider.setMaximum(100)
        self.cizgiKalinligiDSlider.setSingleStep(0.1)
        self.cizgiKalinligiDSlider.setValue(self.cizgiKalinligi * 10)
        self.cizgiKalinligiDSlider.degerDegisti.connect(lambda x: self.cizgiKalinligiDegisti.emit(x))
        self.cizgiKalinligiDSlider.hide()

        self.renkSecici = RenkSeciciWidget(self.renk, boyut=64, parent=self)
        self.renkSecici.setFocusPolicy(Qt.FocusPolicy.TabFocus)
        self.renkSecici.renkDegisti.connect(self.renkGuncelle)

        temelW = QWidget(self)
        anaLay = QVBoxLayout(temelW)

        anaLay.addLayout(radioLay)
        anaLay.addWidget(self.renkSecici)
        anaLay.addSpacing(5)
        anaLay.addWidget(self.cizgiKalinligiDSlider)
        anaLay.addStretch()

        self.setMinimumWidth(100)
        self.setMinimumHeight(20)
        self.setMinimumSize(300, 190)
        # self.setMinimumSize(280, 165)
        temelW.setMinimumSize(280, 165)
        self.adjustSize()

        self.ekleWidget(temelW)

    # ---------------------------------------------------------------------
    def act_radio_secim_degisti(self, btn):
        if btn == self.radioArkaplan:
            # renk = self.arkaPlanRengi
            if self.parent().cScene.activeItem:
                renk = self.parent().cScene.activeItem.arkaPlanRengi
            else:
                renk = self.parent().cScene.aktifArac.arkaPlanRengi
            self.renk_tipi = "a"
            self.cizgiKalinligiDSlider.hide()
        elif btn == self.radioYazi:
            # renk = self.yaziRengi
            if self.parent().cScene.activeItem:
                renk = self.parent().cScene.activeItem.yaziRengi
            else:
                renk = self.parent().cScene.aktifArac.yaziRengi
            self.renk_tipi = "y"
            self.cizgiKalinligiDSlider.hide()

        elif btn == self.radioCizgi:
            # renk = self.cizgiRengi
            if self.parent().cScene.activeItem:
                renk = self.parent().cScene.activeItem.cizgiRengi
            else:
                renk = self.parent().cScene.aktifArac.cizgiRengi
            self.renk_tipi = "c"
            self.cizgiKalinligiDSlider.show()

        self.renkSecici.disardan_renk_gir(renk)

    # ---------------------------------------------------------------------
    def renkGuncelle(self, renk):
        if self.renk_tipi == "a":
            self.arkaPlanRengi = renk
            self.arkaPlanRengiDegisti.emit(renk)
        elif self.renk_tipi == "y":
            self.yaziRengi = renk
            self.yaziRengiDegisti.emit(renk)
        elif self.renk_tipi == "c":
            self.cizgiRengi = renk
            self.cizgiRengiDegisti.emit(renk)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'defter'
__date__ = '6/25/22'
__author__ = 'E. Y.'

import sys
from PySide6.QtCore import Signal, QPoint
from PySide6.QtGui import QIcon, Qt, QColor, QLinearGradient, QGradient, QPainter, QBrush, QPen
from PySide6.QtWidgets import QApplication, QWidget, QVBoxLayout, QHBoxLayout
from .renkCubuguVeSayiKutusu import RenkCubuguVeSayiKutusu
from .shared import kutulu_arkaplan_olustur


#######################################################################
class RenkSecilenKareW(QWidget):
    renkDegisti = Signal(QColor)

    # ---------------------------------------------------------------------
    def __init__(self, eskiRenk, parent=None):
        super(RenkSecilenKareW, self).__init__(parent)

        self.x_oran = self.width() / 255
        self.y_oran = self.height() / 255
        self.halkaKoordiant = QPoint()

        self.renkTon = eskiRenk.hsvHue()
        self.seffalik = eskiRenk.alpha()
        self.renkleriAyarla()
        self.halkayi_koordianata_tasi(eskiRenk.hsvSaturation(), eskiRenk.value())

    # ---------------------------------------------------------------------
    def resizeEvent(self, event):
        self.x_oran = self.width() / 255
        self.y_oran = self.height() / 255

        self.renkleriAyarla()
        # self.renkDegisimiBildir()
        # self.update()
        super(RenkSecilenKareW, self).resizeEvent(event)

    # ---------------------------------------------------------------------
    def renkleriAyarla(self):
        self.renkGrad = QLinearGradient(0, 0, 1, 0)
        self.renkGrad.setCoordinateMode(QLinearGradient.ObjectMode)
        self.renkGrad.setSpread(QGradient.PadSpread)
        self.renkGrad.setColorAt(0, QColor(255, 255, 255))
        self.renkGrad.setColorAt(1, QColor.fromHsv(self.renkTon, 255, 255))

        self.beyazlikGrad = QLinearGradient(0, 0, 0, 1)
        self.beyazlikGrad.setCoordinateMode(QLinearGradient.ObjectMode)
        self.beyazlikGrad.setSpread(QGradient.PadSpread)
        self.beyazlikGrad.setColorAt(0, QColor(0, 0, 0, 0))
        self.beyazlikGrad.setColorAt(1, QColor(0, 0, 0, 255))
        self.update()

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):
        self.halkaKoordiant = self.koordinat_normalize_et(event.position().x(), event.position().y())

        self.update()
        self.renkDegisimiBildir()

        super(RenkSecilenKareW, self).mousePressEvent(event)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):
        self.halkaKoordiant = self.koordinat_normalize_et(event.position().x(), event.position().y())

        self.update()
        self.renkDegisimiBildir()

    # ---------------------------------------------------------------------
    def halkayi_koordianata_tasi(self, doygunluk, beyazlik):
        x = self.x_oran * doygunluk
        y = self.width() - (beyazlik * self.y_oran)

        self.halkaKoordiant = self.koordinat_normalize_et(x, y)
        self.update()

    # ---------------------------------------------------------------------
    def koordinat_normalize_et(self, x, y):
        # self.halkaKoordiant = QPoint(0 if x < 0 else x if x < self.width() else self.width(),
        #                              0 if y < 0 else y if y < self.height() else self.height())

        x = min(max(x, 0), self.width())
        y = min(max(y, 0), self.height())

        return QPoint(x, y)

    # ---------------------------------------------------------------------
    def renk_normalize_et(self, doygunluk, beyazlik):
        doygunluk = min(max(doygunluk, 0), 255)
        beyazlik = min(max(beyazlik, 0), 255)

        renk = QColor.fromHsv(self.renkTon, doygunluk, beyazlik, self.seffalik)

        return renk

    # ---------------------------------------------------------------------
    def yazRenkTonDisardan(self, renkTon):
        self.renkTon = renkTon
        self.renkleriAyarla()

    # ---------------------------------------------------------------------
    def yazSeffaflikDisardan(self, alpha):
        self.seffalik = alpha

    # ---------------------------------------------------------------------
    def yazRenkTon(self, renkTon):
        """renkSecici ton cubugu hareketinden geliyor"""
        self.renkTon = renkTon
        self.renkleriAyarla()
        self.renkDegisimiBildir()

    # ---------------------------------------------------------------------
    def yazSeffaflik(self, alpha):
        """renkSecici seffaflik cubugu hareketinden geliyor"""
        self.seffalik = alpha
        self.renkDegisimiBildir()

    # ---------------------------------------------------------------------
    def renkDegisimiBildir(self):
        doygunluk = self.halkaKoordiant.x() / self.x_oran
        beyazlik = 255 - (self.halkaKoordiant.y() / self.y_oran)

        renk = self.renk_normalize_et(doygunluk, beyazlik)
        self.renkDegisti.emit(renk)

    # ---------------------------------------------------------------------
    def paintEvent(self, event):
        p = QPainter(self)
        # p.begin()
        p.fillRect(self.rect(), QBrush(self.renkGrad))
        p.fillRect(self.rect(), QBrush(self.beyazlikGrad))

        p.setBrush(Qt.BrushStyle.NoBrush)
        p.setPen(QPen(Qt.GlobalColor.white, 1))
        p.drawEllipse(self.halkaKoordiant, 7, 7)

        p.end()


#######################################################################
class RenkGostergeW(QWidget):

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(RenkGostergeW, self).__init__(parent)

        self.renk = QColor(0, 0, 0, 0)

    # ---------------------------------------------------------------------
    def renkGuncelle(self, renk):
        self.renk = renk
        self.update()

    # ---------------------------------------------------------------------
    def paintEvent(self, event):
        p = QPainter(self)
        # p.begin()
        p.fillRect(self.rect(), QBrush(self.renk))
        p.end()


#######################################################################
class RenkSeciciWidget(QWidget):
    renkDegisti = Signal(QColor)

    # ---------------------------------------------------------------------
    def __init__(self, eskiRenk, boyut=256, parent=None):
        super(RenkSeciciWidget, self).__init__(parent)
        # self.setMinimumWidth(250)
        # self.setMinimumHeight(170)
        self.eskiRenk = eskiRenk.toHsv()

        # --------------------
        self.rTonVeSayiKutusu = RenkCubuguVeSayiKutusu(self.tr("H"), Qt.Orientation.Horizontal, self)
        # self.rTonVeSayiKutusu.setFixedWidth(150)
        self.rTonVeSayiKutusu.setRange(0, 359)

        # ########  TON ARKAPLAN  ################
        gradient = QLinearGradient(0, 0, 1, 0)
        gradient.setCoordinateMode(QLinearGradient.CoordinateMode.ObjectMode)
        gradient.setColorAt(0, QColor.fromHsv(0, 255, 255))
        gradient.setColorAt(0.16105497, QColor.fromHsv(60, 255, 255))
        gradient.setColorAt(0.35173747, QColor.fromHsv(120, 255, 255))
        gradient.setColorAt(0.48789391, QColor.fromHsv(180, 255, 255))
        # gradient.setColorAt(0.48789391, renkHsv)
        gradient.setColorAt(0.70091939, QColor.fromHsv(240, 255, 255))
        gradient.setColorAt(0.83720928, QColor.fromHsv(300, 255, 255))
        gradient.setColorAt(1, QColor.fromHsv(359, 255, 255))
        self.rTonVeSayiKutusu.setBgGradient(gradient)
        self.rTonVeSayiKutusu.degerDegisti.connect(self.renk_ton_degisti_slider)

        # --------------------
        self.rSeffaflikVeSayiKutusu = RenkCubuguVeSayiKutusu(self.tr("A"), Qt.Orientation.Horizontal, self)
        # self.rSeffaflikVeSayiKutusu.setFixedWidth(150)
        self.rSeffaflikVeSayiKutusu.setRange(0, 255)
        kutulu_arkaplan_olustur(self.rSeffaflikVeSayiKutusu)
        self.rSeffaflikVeSayiKutusu.degerDegisti.connect(self.renk_seffaflik_degisti_slider)

        # --------------------
        self.renkSecmeKutusu = RenkSecilenKareW(self.eskiRenk, self)
        self.renkSecmeKutusu.setFocusPolicy(Qt.FocusPolicy.TabFocus)
        self.renkSecmeKutusu.setFixedWidth(boyut)
        self.renkSecmeKutusu.setFixedHeight(boyut)
        self.renkSecmeKutusu.renkDegisti.connect(self.renk_degisimi_bildir)

        # --------------------
        self.anaLay = QVBoxLayout()
        self.setLayout(self.anaLay)
        vLay = QVBoxLayout()
        hLay = QHBoxLayout()
        vLay.addWidget(self.rTonVeSayiKutusu)
        vLay.addWidget(self.rSeffaflikVeSayiKutusu)
        hLay.addWidget(self.renkSecmeKutusu)
        hLay.addLayout(vLay)
        self.anaLay.addLayout(hLay)
        self.renk = eskiRenk

        self.disardan_renk_gir(self.eskiRenk)
        self.setFocus()

    # ---------------------------------------------------------------------
    def disardan_renk_gir(self, renk):
        # self.eskiRenk = renk.toHsv()
        self.eskiRenk = renk
        self.rTonVeSayiKutusu.setValue(self.eskiRenk.hsvHue())
        # self.seffalik_cubugu_arkaplan_degistir(self.eskiRenk.hsvHue())
        self.rSeffaflikVeSayiKutusu.setValue(self.eskiRenk.alpha())
        self.renk_ton_degisti_disardan(self.eskiRenk.hsvHue())
        self.renk_seffaflik_degisti_disardan(self.eskiRenk.alpha())
        self.renkSecmeKutusu.halkayi_koordianata_tasi(self.eskiRenk.saturation(), self.eskiRenk.value())

    # ---------------------------------------------------------------------
    def renk_degisimi_bildir(self, renk):
        self.renk = renk
        self.renkDegisti.emit(renk)

    # ---------------------------------------------------------------------
    def renk_ton_degisti_disardan(self, hue):
        self.renkSecmeKutusu.yazRenkTonDisardan(hue)
        self.seffalik_cubugu_arkaplan_degistir(hue)
        self.rTonVeSayiKutusu.blockSignals(True)
        self.rTonVeSayiKutusu.setValue(hue)
        self.rTonVeSayiKutusu.blockSignals(False)

    # ---------------------------------------------------------------------
    def renk_seffaflik_degisti_disardan(self, alpha):
        self.renkSecmeKutusu.yazSeffaflikDisardan(alpha)
        self.rSeffaflikVeSayiKutusu.blockSignals(True)
        self.rSeffaflikVeSayiKutusu.setValue(alpha)
        self.rSeffaflikVeSayiKutusu.blockSignals(False)

    # ---------------------------------------------------------------------
    def renk_ton_degisti_slider(self, hue):
        self.renkSecmeKutusu.yazRenkTon(hue)
        self.seffalik_cubugu_arkaplan_degistir(hue)

    # ---------------------------------------------------------------------
    def renk_seffaflik_degisti_slider(self, alpha):
        self.renkSecmeKutusu.yazSeffaflik(alpha)

    # ---------------------------------------------------------------------
    def seffalik_cubugu_arkaplan_degistir(self, renk_tonu):
        # ########  SEFFAFLIK ARKAPLAN  ################
        gradient = QLinearGradient(0, 0, 1, 0)
        gradient.setCoordinateMode(QLinearGradient.CoordinateMode.ObjectMode)

        gradient.setColorAt(0, QColor.fromHsv(renk_tonu, 255, 255, 0))
        gradient.setColorAt(1, QColor.fromHsv(renk_tonu, 255, 255, 255))

        self.rSeffaflikVeSayiKutusu.setBgGradient(gradient)


# ---------------------------------------------------------------------
def calistir():
    prog = QApplication(sys.argv)
    prog.setWindowIcon(QIcon(":icons/defter.png"))
    prog.setStyle("fusion")

    pencere = RenkSeciciWidget(QColor(100, 150, 120, 128),
                               )
    pencere.show()

    sys.exit(prog.exec())


# ---------------------------------------------------------------------
if __name__ == "__main__":
    calistir()



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '8/1/15'

from PySide6.QtCore import (Qt, QRect, Signal, QSize, QTimer)
from PySide6.QtGui import QPainter
from PySide6.QtWidgets import (QRubberBand, QApplication, QLabel)


#######################################################################
class MyRubberBand(QRubberBand):
    rubberBandHidden = Signal()

    # ---------------------------------------------------------------------
    def __init__(self, shape=QRubberBand.Shape.Rectangle, parent=None):
        super(MyRubberBand, self).__init__(shape, parent)
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.setWindowFlags(Qt.WindowType.FramelessWindowHint | Qt.WindowType.WindowStaysOnTopHint)
        # defaultFlags = self.windowFlags()
        # self.setWindowFlags(defaultFlags | Qt.ToolTip)
        # self.setWindowFlags(Qt.ToolTip)
        # self.setStyleSheet("selection-background-color: transparent")

    # ---------------------------------------------------------------------
    def hideEvent(self, event):
        super(MyRubberBand, self).hideEvent(event)
        QApplication.processEvents()
        QTimer.singleShot(50, self.rubberBandHidden.emit)


#######################################################################
class TamEkranWidget_CM_Off(QLabel):
    rubberBandReleased = Signal(QRect)
    esc_key_pressed = Signal()

    # ---------------------------------------------------------------------
    # def __init__(self, texts, parent=None):
    def __init__(self, parent=None):

        super(TamEkranWidget_CM_Off, self).__init__(parent)
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.setAttribute(Qt.WidgetAttribute.WA_TranslucentBackground, True)

        self.setWindowFlags(Qt.WindowType.FramelessWindowHint | Qt.WindowType.WindowStaysOnTopHint)

        # self.setAttribute(Qt.WA_PaintOnScreen, True)
        self.setAutoFillBackground(True)
        # self.setStyleSheet("background-color:none;")
        # self.setAutoFillBackground(True)

        # self.rubberBand = QRubberBand(QRubberBand.Rectangle, self)
        # self.rubberBand = MyRubberBand(QRubberBand.Rectangle, self)
        self.rubberBand = MyRubberBand(QRubberBand.Shape.Rectangle)
        self.rubberBand.rubberBandHidden.connect(self.act_rubberband_hidden)

        self.alan = QRect()

        self.screen = QApplication.primaryScreen()

        # self.setAutoFillBackground(True)

        # self.setWindowOpacity(0.01)
        # self.setWindowOpacity(0)
        # self.setStyleSheet("QWidget{background:transparent;}")
        # self.setStyleSheet("QWidget{background-color:none;}")

        self.showFullScreen()

    # ---------------------------------------------------------------------
    def act_rubberband_hidden(self):
        # QApplication.processEvents()
        QTimer.singleShot(0, lambda: self.rubberBandReleased.emit(self.alan))

    # ---------------------------------------------------------------------
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.drawPixmap(self.rect(), self.screen.grabWindow(0))
        # super(TamEkranWidget_CM_Off, self).paintEvent(event)

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):
        self.origin = event.pos()

        self.rubberBand.setGeometry(QRect(self.origin, QSize()))
        self.rubberBand.show()

        super(TamEkranWidget_CM_Off, self).mousePressEvent(event)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):

        self.rubberBand.setGeometry(QRect(self.origin, event.pos()).normalized())

        # self.repaint()
        super(TamEkranWidget_CM_Off, self).mouseMoveEvent(event)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        self.alan = self.rubberBand.geometry()
        self.rubberBand.hide()

        # super(TamEkranWidget_CM_Off, self).mouseReleaseEvent(event)
        # return QWidget.mouseReleaseEvent(event)

    # ---------------------------------------------------------------------
    def keyPressEvent(self, event):

        if event.key() == Qt.Key.Key_Escape:
            self.esc_key_pressed.emit()
        super(TamEkranWidget_CM_Off, self).keyPressEvent(event)
        self.close()
        self.deleteLater()



# -*- coding: utf-8 -*-
# .

__project_name__ = 'defter'
__date__ = '10/13/23'
__author__ = 'E. Y.'

from PySide6.QtGui import QPainter, QFontMetrics
from PySide6.QtWidgets import QLabel


#######################################################################
class DikeyEtiket(QLabel):

    # ---------------------------------------------------------------------
    def __init__(self, *args, dikey=True):
        super(DikeyEtiket, self).__init__(*args)
        self.dikey = dikey

    # ---------------------------------------------------------------------
    def paintEvent(self, event):
        if self.dikey:
            painter = QPainter(self)
            painter.translate(0, self.height())
            painter.rotate(-90)
            # calculate the size of the font
            fm = QFontMetrics(painter.font())
            xoffset = int(fm.boundingRect(self.text()).width() / 2)
            yoffset = int(fm.boundingRect(self.text()).height() / 2)
            x = int(self.width() / 2) + yoffset
            y = int(self.height() / 2) - xoffset
            # because we rotated the label, x affects the vertical placement, and y affects the horizontal
            painter.drawText(y, x, self.text())
            painter.end()
        else:
            super(DikeyEtiket, self).paintEvent(event)

    # ---------------------------------------------------------------------
    def minimumSizeHint(self):
        if self.dikey:
            return QLabel.minimumSizeHint(self).transposed()
        else:
            return QLabel.minimumSizeHint(self)

    # ---------------------------------------------------------------------
    def sizeHint(self):
        if self.dikey:
            return QLabel.sizeHint(self).transposed()
        else:
            return QLabel.sizeHint(self)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '23/Aug/2016'

from PySide6.QtCore import Qt, QRegExp
from PySide6.QtGui import QSyntaxHighlighter, QTextCharFormat, QFont


########################################################################
class HtmlHighlighter(QSyntaxHighlighter):
    def __init__(self, parent=None):
        super(HtmlHighlighter, self).__init__(parent)

        keywordFormat = QTextCharFormat()
        keywordFormat.setForeground(Qt.darkBlue)
        keywordFormat.setFontWeight(QFont.Bold)

        keywordPatterns = ["\\b?xml\\b", "/>", ">", "<"]

        self.highlightingRules = [(QRegExp(pattern), keywordFormat)
                                  for pattern in keywordPatterns]

        htmlElementFormat = QTextCharFormat()
        htmlElementFormat.setFontWeight(QFont.Bold)
        htmlElementFormat.setForeground(Qt.darkMagenta)
        # self.highlightingRules.append((QRegExp("\\b[A-Za-z0-9_]+(?=[\s/>])"), htmlElementFormat))
        # self.highlightingRules.append((QRegExp("\\b<([A-Z][A-Z0-9]*)\b[^>]*>(.*?)</\1>"), htmlElementFormat))
        self.highlightingRules.append((QRegExp(''), htmlElementFormat))

        htmlAttributeFormat = QTextCharFormat()
        htmlAttributeFormat.setFontItalic(True)
        htmlAttributeFormat.setForeground(Qt.blue)
        self.highlightingRules.append((QRegExp("\\b[A-Za-z0-9_]+(?=\\=)"), htmlAttributeFormat))

    # VIRTUAL FUNCTION WE OVERRIDE THAT DOES ALL THE COLLORING
    def highlightBlock(self, text):

        # for every pattern
        for pattern, frmt in self.highlightingRules:

            # Create a regular expression from the retrieved pattern
            expression = QRegExp(pattern)

            # Check what index that expression occurs at with the ENTIRE text
            index = expression.indexIn(text)

            # While the index is greater than 0
            while index >= 0:
                # Get the length of how long the expression is true,
                # set the format from the start to the length with the text format
                length = expression.matchedLength()
                self.setFormat(index, length, frmt)

                # Set index to where the expression ends in the text
                index = expression.indexIn(text, index + length)

        # HANDLE QUOTATION MARKS NOW.. WE WANT TO START WITH " AND END WITH "..
        # A THIRD " SHOULD NOT CAUSE THE WORDS INBETWEEN SECOND AND THIRD TO BE COLORED
        self.setCurrentBlockState(0)
